module Main where

import System.Environment

import CommandLineInterface
import Input
import Output
import Result
import Weaver

main :: IO ()
main = do
  args <- getArgs
  parseArgsAndProceed args

parseArgsAndProceed :: [String] -> IO ()
parseArgsAndProceed args =
  case cli args of
    Error e -> processErr e
    Value s -> procedure (inputFile s) (outputFile s) (showHelp s)

procedure :: String -> String -> Bool -> IO ()
procedure _ _ True   = printHelp
procedure inF outF _ = do
  c <- readFile inF
  case weaved c of
    Error e -> processErr e
    Value ls -> processVal ls
  where
    weaved     = weave . preprocess . lines
    processVal = writeFile outF . unlines . postprocess

processErr = putStrLn
printHelp  = putStrLn "Usage: maudeaop [-o OUTPUT] SOURCE" >>
             putStrLn "  or:  maudeaop -h" >>
             putStrLn "Performes weaving of aspects." >>
             putStrLn "" >>
             putStrLn "The very only mandatory argument is SOURCE which specifies the source file" >>
             putStrLn "where the aspects and operations are defined." >>
             putStrLn "There are following optional switches:" >>
             putStrLn "  -o <file>                Place the output into <file>; `a.maude` is used" >>
             putStrLn "                           as default if the switch is ommitted" >>
             putStrLn "  -h                       Display this information"


