module AspectContext where 

import Control.Monad(filterM)

import Parser
import PointcutMatcher
import Result
import Utils

-- | Structure for a single aspect
-- (a pair of aspect sentence and advice sentences)
type Aspect = (AspectSen, [AdviceSen])

-- | Takes a list of all aspect sentences and a list of all advice sentences.
-- Pairs them together and returns list of aspects.
buildAspectContext :: [AspectSen] -> [AdviceSen] -> Result [Aspect]
buildAspectContext asps advs = foldResult (pairAspectAdvices advs) asps

-- | Expects a list of advices and a single aspect. It searches in the list
-- for all advices belonging to the aspect.
pairAspectAdvices :: [AdviceSen] -> AspectSen -> Result Aspect
pairAspectAdvices advs asp =
  case filter (matchAspectAdvice asp) advs of
    []       -> Error ("No advices found for aspect " ++ aspectName asp)
    matching -> Value (asp, matching)


-- | Returns true if the given aspect and advice match.
-- I.e. the given advice belongs to the given aspect.
matchAspectAdvice :: AspectSen -> AdviceSen -> Bool
matchAspectAdvice (AspectSen asName asInCount _) (AdviceSen adAspect adInputs _) =
  (asName == adAspect) && (asInCount == length adInputs)

-- | Takes Aspect context, a declaration of an operation
-- and returns all the appropriate aspects (checks the pointcuts).
searchAspectContext :: [Aspect] -> String -> [Aspect]
searchAspectContext [] _      = []
searchAspectContext asCxt op = filter matcher asCxt
  where
    matcher (as, _) = matchPointcut op $ aspectPointcut as