module OperationContext where

import Data.List(find)

import Input
import Parser
import PointcutMatcher
import Result

-- | Takes a list of aspect sentences and a list of operations (raw lines
-- from source code). Then it parses declaration of each operation and
-- creates an operation context which contains all the operations that has
-- at least one matching aspect.
buildOperationContext :: [AspectSen] -> [String] -> Result [String]
buildOperationContext asps ops = Value (filter (hasAspect asps) $ map opDeclaration ops)
  where
    hasAspect :: [AspectSen] -> String -> Bool
    hasAspect [] op                         = False
    hasAspect (a:as) op
      | matchPointcut op (aspectPointcut a) = True
      | otherwise                           = hasAspect as op

-- | Takes a list of operation declarations (operation context) and an
-- equation. Returns operation declaration that belongs to the equation.
searchOperationContext :: [String] -> String -> Maybe String
searchOperationContext cxt s = find (matchOperation s) cxt
  where
    matchOperation :: String -> String -> Bool
    matchOperation eq op
      | length eqD /= length opD = False
      | otherwise                = all equals zipped
      where
        opD    = words op
        eqD    = words $ eqDeclaration eq
        zipped = zip opD eqD
        equals ("_", _) = True
        equals (a, b)   = a == b