module Utils (
  split,
  strip,
  isAlphaNum
  ) where

import qualified Data.Char(isSpace, isAlphaNum)

-- | Removes spaces from the beginning and from the end of the
-- given string.
strip :: String -> String
strip = stripL . stripR

-- | Removes spaces from the beginning of the given string.
stripL :: String -> String
stripL = dropWhile Data.Char.isSpace

-- | Removes spaces from the end of the given string.
stripR :: String -> String
stripR = reverse . stripL . reverse

-- | Tests whether the given string contains only AlphaNum
-- chars.
isAlphaNum :: String -> Bool
isAlphaNum [] = True
isAlphaNum (x:xs)
  | Data.Char.isAlphaNum x = isAlphaNum xs
  | otherwise              = False

-- | Splits the list at delimiters defined by the inputting function
-- while keeping the delimiters in output
split :: (a -> Bool) -> [a] -> [[a]]
split f s = split2 f s []
  where
    split2 :: (a -> Bool) -> [a] -> [a] -> [[a]]
    split2 _ [] []      = []
    split2 _ [] acc     = [acc]
    split2 f (x:xs) acc
      | d && null acc   = [x] : split2 f xs []
      | d               = acc : ([x] : split2 f xs [])
      | otherwise       = split2 f xs (acc ++ [x])
      where
        d = f x