module Weaver where

import AdviceApplicator
import AspectContext
import OperationContext
import Input
import Parser
import Result

-- | Takes a list of lines of source code.
-- Builds aspect & operation context, realizes transformation (i. e. weaving)
-- and returns the result.
weave :: [String] -> Result [String]
weave ls = do
  aspects          <- parseAspects aspects'
  advices          <- parseAdvices advices'
  aspectContext    <- buildAspectContext aspects advices
  operationContext <- buildOperationContext aspects operations
  foldResult (transform aspectContext operationContext) ls
  where
    aspects'   = filter isAspect ls
    advices'   = filter isAdvice ls
    operations = filter isOperation ls
    equations  = filter isEquation ls

-- | Takes aspect context, operation context and a sentence as parameters.
-- Returns the sentence after all relevant aspects were applied.
transform :: [Aspect] -> [String] -> String -> Result String
transform aspCxt opCxt s
  | isAspect s   = Value ""
  | isAdvice s   = Value ""
  | isEquation s = transformEquation aspCxt opCxt s
  | otherwise    = Value s

-- | Takes an aspect context, operation context and an equation to transform.
-- Returns the result.
transformEquation :: [Aspect] -> [String] -> String -> Result String
transformEquation aspCxt opCxt s =
  case searchOperationContext opCxt s of
    Nothing -> Value s
    Just op -> applyAdvices op s (advices op)
  where
    aspects = searchAspectContext aspCxt
    advices = concatMap snd . aspects

