module AdviceApplicatorTest where

import Test.HUnit

import AdviceApplicator
import Parser
import Result

tests = TestList[
  TestLabel "FindVariablesTest" testFindVariables,
  TestLabel "MapVariablesTest" testMapVariables,
  TestLabel "ApplyMappingTest" testApplyMapping,
  TestLabel "RewriteEquationTest" testRewriteEquation,
  TestLabel "ApplyAdviceTest" testApplyAdvice,
  TestLabel "ApplyAdvicesTest" testApplyAdvices]

testFindVariables = TestCase(do
  assertEqual "testing variable finding" (Value ["a", "b"]) $ findVariables "foo _ _" "foo a b"
  assertEqual "testing variable finding" (Value ["a", "b"]) $ findVariables "_ foo _" "a foo b"
  assertEqual "testing variable finding" (Value ["a", "b", "c"]) $ findVariables "_ foo _ bar _" "a foo b bar c"
  assertEqual "testing variable finding" (Value ["a", "b"]) $ findVariables "f _ _" "f a b"
  assertEqual "testing variable finding" (Error "") $ findVariables "_ foo _ bar _" "a foo b bar a c"
  assertEqual "testing variable finding" (Error "") $ findVariables "foo _ _" "bar a b")

testMapVariables = TestCase(do
  assertEqual "variable mapping" (Value [("x", "a"), ("y", "b")]) $ mapVariables "foo _ _" "foo a b" (AdviceSen "as" ["x", "y"] "")
  assertEqual "variable mapping" (Value [("x", "a"), ("y", "b")]) $ mapVariables "_ foo _" "a foo b" (AdviceSen "as" ["x", "y"] "")
  assertEqual "variable mapping" (Value [("x", "a"), ("y", "b")]) $ mapVariables "f _ _" "f a b" (AdviceSen "as" ["x", "y"] "x = (x * x + y)")
  assertEqual "variable mapping" (Error "") $ mapVariables "_ foo _" "a foo b" (AdviceSen "as" ["x", "y", "z"] ""))

testApplyMapping = TestCase(do
  assertEqual "testing mapping application" (Value "a = a .") $ applyMapping [("x", "a")] "x = x ."
  assertEqual "testing mapping application" (Value "a = a plus b .") $ applyMapping [("x", "a"), ("y", "b")] "x = x plus y ."
  assertEqual "testing mapping application" (Value "a = ( a * a + b ) .") $ applyMapping [("x", "a"), ("y", "b")] "x = (x * x + y) .")

testRewriteEquation = TestCase(assertEqual "testing equation rewriting" "eq foo a b = 2 + b ." $ rewriteEquation "eq foo a b = a + b ." "a = 2")

testApplyAdvice = TestCase(do
  assertEqual "testing advice application" (Value "eq a foo b = b + b .") $ applyAdvice "_ foo _" "eq a foo b = a + b ." (AdviceSen "as" ["x", "y"] "x = y")
  assertEqual "testing advice application" (Value "eq foo a b c = b + c .") $ applyAdvice "foo _ _ _" "eq foo a b c = a ." (AdviceSen "as" ["x", "y", "z"] "x = y + z"))

testApplyAdvices = TestCase(do
  assertEqual "testing advices application" (Value "eq a foo b = c .") $ applyAdvices "_ foo _" "eq a foo b = a ." [AdviceSen "as" ["x", "y"] "x = y", AdviceSen "as" ["x", "y"] "y = c"]
  assertEqual "testing advices application" (Error "") $ applyAdvices "foo _" "eq a foo b = a ." [AdviceSen "as" ["x", "y"] "x = y", AdviceSen "as" ["x", "y"] "y = c"])