module ResultTest where

import Test.HUnit

import Result

tests = TestList [
  TestLabel "FromValueTest" testFromValue,
  TestLabel "IsValueTest" testIsValue,
  TestLabel "IsErrorTest" testIsError,
  TestLabel "FoldResultTest" testFoldResult,
  TestLabel "ResultInstancesTest" testResultInstances]

testFromValue = TestCase(do
  assertEqual "test fromValue" 3 $ fromValue (Value 3)
  assertEqual "test fromValue" "test" $ fromValue (Value "test"))

testIsValue = TestCase(do
  assertBool "test isValue" $ isValue (Value 3)
  assertBool "test isValue" $ not $ isValue (Error ""))

testIsError = TestCase(do
  assertBool "test isError" $ not $ isError (Value 3)
  assertBool "test isError" $ isError (Error ""))

f 2 = Error ""
f x = Value x

testFoldResult = TestCase(do
  assertEqual "test foldResult" (Error "") $ foldResult f [0, 1, 2, 3, 4, 5]
  assertEqual "test foldResult" (Error "") $ foldResult f [2, 3, 4, 5]
  assertEqual "test foldResult" (Error "") $ foldResult f [3, 4, 5, 2]
  assertEqual "test foldResult" (Value [1, 3, 4]) $ foldResult f [1, 3, 4]
  assertEqual "test foldResult" (Value []) $ foldResult f [])

testResultInstances = TestCase (do
  assertBool "test Eq" $ Value "foo" == Value "foo"
  assertBool "test Eq" $ Error "foo" == (Error "bar" :: Result Int)
  assertBool "test Eq" $ Error "foo" /= Value "bar"
  assertBool "test Eq" $ Value "foo" /= Error "bar"
  assertBool "test Eq" $ Value "foo" /= Value "bar"
  assertEqual "test Show" "\"foo\"" $ show (Value "foo")
  assertEqual "test Show" "\"foo\"" $ show (Error "foo" :: Result Int)
  assertEqual "test fmap" (Value 6) $ fmap (*2) (Value 3)
  assertEqual "test fmap" (Error "foo") $ fmap (*2) (Error "foo")
  assertEqual "test <*>" (Value 6) $ Value (*2) <*> Value 3
  assertEqual "test <*>" (Error "foo") $ (Error "foo" :: Result (Int -> Int)) <*> Value 3
  assertEqual "test fail" (Error "") (fail "failed" :: Result Int))